import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { UsersRepository } from '../../adapters/secondaries/users.repository';
import { UpdateUserCommand } from '../../commands/update-user.command';
import { UpdateUserRequest } from '../dtos/update-user.request';
import { User } from '../entities/user';

@CommandHandler(UpdateUserCommand)
export class UpdateUserUseCase {
  constructor(
    private readonly repository: UsersRepository,
    private readonly messager: Messager,
    @InjectMapper() private readonly mapper: Mapper,
  ) {}

  execute = async (command: UpdateUserCommand): Promise<User> => {
    const entity = this.mapper.map(
      command.updateUserRequest,
      UpdateUserRequest,
      User,
    );

    try {
      const user = await this.repository.update(
        command.updateUserRequest.uuid,
        entity,
      );
      this.messager.publish(
        'user.update',
        JSON.stringify(command.updateUserRequest),
      );
      this.messager.publish(
        'logging.user.update.info',
        JSON.stringify(command.updateUserRequest),
      );
      return user;
    } catch (error) {
      this.messager.publish(
        'logging.user.update.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  };
}
